@@ -1,13 +1,13 @@
 """Tools for helping with ANSI color codes."""
-import re
+import sys
 import string
 import warnings
+import builtins
 
-from xonsh.lazyasd import LazyObject, LazyDict
-
-
-RE_BACKGROUND = LazyObject(lambda: re.compile('(bg|bg#|bghex|background)'),
-                           globals(), 'RE_BACKGROUND')
+from xonsh.platform import HAS_PYGMENTS
+from xonsh.lazyasd import LazyDict
+from xonsh.color_tools import (RE_BACKGROUND, BASE_XONSH_COLORS, make_palette,
+                               find_closest_color, rgb2short, rgb_to_256)
 
 
 def ansi_partial_color_format(template, style='default', cmap=None, hide=False):
@@ -19,7 +19,7 @@ def ansi_partial_color_format(template, style='default', cmap=None, hide=False):
     template : str
         The template string, potentially with color names.
     style : str, optional
-        Sytle name to look up color map from.
+        Style name to look up color map from.
     cmap : dict, optional
         A color map to use, this will prevent the color map from being
         looked up via the style name.
@@ -44,9 +44,13 @@ def _ansi_partial_color_format_main(template, style='default', cmap=None, hide=F
     elif style in ANSI_STYLES:
         cmap = ANSI_STYLES[style]
     else:
-        msg = 'Could not find color style {0!r}, using default.'.format(style)
-        warnings.warn(msg, RuntimeWarning)
-        cmap = ANSI_STYLES['default']
+        try:  # dynamically loading the style
+            cmap = ansi_style_by_name(style)
+        except Exception:
+            msg = 'Could not find color style {0!r}, using default.'
+            print(msg.format(style), file=sys.stderr)
+            builtins.__xonsh_env__['XONSH_COLOR_STYLE'] = 'default'
+            cmap = ANSI_STYLES['default']
     formatter = string.Formatter()
     esc = ('\001' if hide else '') + '\033['
     m = 'm' + ('\002' if hide else '')
@@ -87,298 +91,6 @@ def _ansi_partial_color_format_main(template, style='default', cmap=None, hide=F
     return ''.join(toks)
 
 
-RGB_256 = LazyObject(lambda: {
-    '000000': '16',
-    '00005f': '17',
-    '000080': '04',
-    '000087': '18',
-    '0000af': '19',
-    '0000d7': '20',
-    '0000ff': '21',
-    '005f00': '22',
-    '005f5f': '23',
-    '005f87': '24',
-    '005faf': '25',
-    '005fd7': '26',
-    '005fff': '27',
-    '008000': '02',
-    '008080': '06',
-    '008700': '28',
-    '00875f': '29',
-    '008787': '30',
-    '0087af': '31',
-    '0087d7': '32',
-    '0087ff': '33',
-    '00af00': '34',
-    '00af5f': '35',
-    '00af87': '36',
-    '00afaf': '37',
-    '00afd7': '38',
-    '00afff': '39',
-    '00d700': '40',
-    '00d75f': '41',
-    '00d787': '42',
-    '00d7af': '43',
-    '00d7d7': '44',
-    '00d7ff': '45',
-    '00ff00': '46',
-    '00ff5f': '47',
-    '00ff87': '48',
-    '00ffaf': '49',
-    '00ffd7': '50',
-    '00ffff': '51',
-    '080808': '232',
-    '121212': '233',
-    '1c1c1c': '234',
-    '262626': '235',
-    '303030': '236',
-    '3a3a3a': '237',
-    '444444': '238',
-    '4e4e4e': '239',
-    '585858': '240',
-    '5f0000': '52',
-    '5f005f': '53',
-    '5f0087': '54',
-    '5f00af': '55',
-    '5f00d7': '56',
-    '5f00ff': '57',
-    '5f5f00': '58',
-    '5f5f5f': '59',
-    '5f5f87': '60',
-    '5f5faf': '61',
-    '5f5fd7': '62',
-    '5f5fff': '63',
-    '5f8700': '64',
-    '5f875f': '65',
-    '5f8787': '66',
-    '5f87af': '67',
-    '5f87d7': '68',
-    '5f87ff': '69',
-    '5faf00': '70',
-    '5faf5f': '71',
-    '5faf87': '72',
-    '5fafaf': '73',
-    '5fafd7': '74',
-    '5fafff': '75',
-    '5fd700': '76',
-    '5fd75f': '77',
-    '5fd787': '78',
-    '5fd7af': '79',
-    '5fd7d7': '80',
-    '5fd7ff': '81',
-    '5fff00': '82',
-    '5fff5f': '83',
-    '5fff87': '84',
-    '5fffaf': '85',
-    '5fffd7': '86',
-    '5fffff': '87',
-    '626262': '241',
-    '6c6c6c': '242',
-    '767676': '243',
-    '800000': '01',
-    '800080': '05',
-    '808000': '03',
-    '808080': '244',
-    '870000': '88',
-    '87005f': '89',
-    '870087': '90',
-    '8700af': '91',
-    '8700d7': '92',
-    '8700ff': '93',
-    '875f00': '94',
-    '875f5f': '95',
-    '875f87': '96',
-    '875faf': '97',
-    '875fd7': '98',
-    '875fff': '99',
-    '878700': '100',
-    '87875f': '101',
-    '878787': '102',
-    '8787af': '103',
-    '8787d7': '104',
-    '8787ff': '105',
-    '87af00': '106',
-    '87af5f': '107',
-    '87af87': '108',
-    '87afaf': '109',
-    '87afd7': '110',
-    '87afff': '111',
-    '87d700': '112',
-    '87d75f': '113',
-    '87d787': '114',
-    '87d7af': '115',
-    '87d7d7': '116',
-    '87d7ff': '117',
-    '87ff00': '118',
-    '87ff5f': '119',
-    '87ff87': '120',
-    '87ffaf': '121',
-    '87ffd7': '122',
-    '87ffff': '123',
-    '8a8a8a': '245',
-    '949494': '246',
-    '9e9e9e': '247',
-    'a8a8a8': '248',
-    'af0000': '124',
-    'af005f': '125',
-    'af0087': '126',
-    'af00af': '127',
-    'af00d7': '128',
-    'af00ff': '129',
-    'af5f00': '130',
-    'af5f5f': '131',
-    'af5f87': '132',
-    'af5faf': '133',
-    'af5fd7': '134',
-    'af5fff': '135',
-    'af8700': '136',
-    'af875f': '137',
-    'af8787': '138',
-    'af87af': '139',
-    'af87d7': '140',
-    'af87ff': '141',
-    'afaf00': '142',
-    'afaf5f': '143',
-    'afaf87': '144',
-    'afafaf': '145',
-    'afafd7': '146',
-    'afafff': '147',
-    'afd700': '148',
-    'afd75f': '149',
-    'afd787': '150',
-    'afd7af': '151',
-    'afd7d7': '152',
-    'afd7ff': '153',
-    'afff00': '154',
-    'afff5f': '155',
-    'afff87': '156',
-    'afffaf': '157',
-    'afffd7': '158',
-    'afffff': '159',
-    'b2b2b2': '249',
-    'bcbcbc': '250',
-    'c0c0c0': '07',
-    'c6c6c6': '251',
-    'd0d0d0': '252',
-    'd70000': '160',
-    'd7005f': '161',
-    'd70087': '162',
-    'd700af': '163',
-    'd700d7': '164',
-    'd700ff': '165',
-    'd75f00': '166',
-    'd75f5f': '167',
-    'd75f87': '168',
-    'd75faf': '169',
-    'd75fd7': '170',
-    'd75fff': '171',
-    'd78700': '172',
-    'd7875f': '173',
-    'd78787': '174',
-    'd787af': '175',
-    'd787d7': '176',
-    'd787ff': '177',
-    'd7af00': '178',
-    'd7af5f': '179',
-    'd7af87': '180',
-    'd7afaf': '181',
-    'd7afd7': '182',
-    'd7afff': '183',
-    'd7d700': '184',
-    'd7d75f': '185',
-    'd7d787': '186',
-    'd7d7af': '187',
-    'd7d7d7': '188',
-    'd7d7ff': '189',
-    'd7ff00': '190',
-    'd7ff5f': '191',
-    'd7ff87': '192',
-    'd7ffaf': '193',
-    'd7ffd7': '194',
-    'd7ffff': '195',
-    'dadada': '253',
-    'e4e4e4': '254',
-    'eeeeee': '255',
-    'ff0000': '196',
-    'ff005f': '197',
-    'ff0087': '198',
-    'ff00af': '199',
-    'ff00d7': '200',
-    'ff00ff': '201',
-    'ff5f00': '202',
-    'ff5f5f': '203',
-    'ff5f87': '204',
-    'ff5faf': '205',
-    'ff5fd7': '206',
-    'ff5fff': '207',
-    'ff8700': '208',
-    'ff875f': '209',
-    'ff8787': '210',
-    'ff87af': '211',
-    'ff87d7': '212',
-    'ff87ff': '213',
-    'ffaf00': '214',
-    'ffaf5f': '215',
-    'ffaf87': '216',
-    'ffafaf': '217',
-    'ffafd7': '218',
-    'ffafff': '219',
-    'ffd700': '220',
-    'ffd75f': '221',
-    'ffd787': '222',
-    'ffd7af': '223',
-    'ffd7d7': '224',
-    'ffd7ff': '225',
-    'ffff00': '226',
-    'ffff5f': '227',
-    'ffff87': '228',
-    'ffffaf': '229',
-    'ffffd7': '230',
-    'ffffff': '231',
-    }, globals(), 'RGB_256')
-
-RE_RGB3 = LazyObject(lambda: re.compile(r'(.)(.)(.)'), globals(), 'RE_RGB3')
-RE_RGB6 = LazyObject(lambda: re.compile(r'(..)(..)(..)'), globals(), 'RE_RGB6')
-
-
-def rgb_to_ints(rgb):
-    """Converts an RGB string into a tuple of ints."""
-    if len(rgb) == 6:
-        return tuple([int(h, 16) for h in RE_RGB6.split(rgb)[1:4]])
-    else:
-        return tuple([int(h*2, 16) for h in RE_RGB3.split(rgb)[1:4]])
-
-
-def rgb_to_256(rgb):
-    """Find the closest ANSI 256 approximation to the given RGB value.
-    Thanks to Micah Elliott (http://MicahElliott.com) for colortrans.py
-    """
-    rgb = rgb.lstrip('#')
-    if len(rgb) == 0:
-        return '0', '000000'
-    incs = (0x00, 0x5f, 0x87, 0xaf, 0xd7, 0xff)
-    # Break 6-char RGB code into 3 integer vals.
-    parts = rgb_to_ints(rgb)
-    res = []
-    for part in parts:
-        i = 0
-        while i < len(incs)-1:
-            s, b = incs[i], incs[i+1]  # smaller, bigger
-            if s <= part <= b:
-                s1 = abs(s - part)
-                b1 = abs(b - part)
-                if s1 < b1:
-                    closest = s
-                else:
-                    closest = b
-                res.append(closest)
-                break
-            i += 1
-    res = ''.join([('%02.x' % i) for i in res])
-    equiv = RGB_256[res]
-    return equiv, res
-
-
 def ansi_color_style_names():
     """Returns an iterable of all ANSI color style names."""
     return ANSI_STYLES.keys()
@@ -1149,3 +861,38 @@ del (_algol_style, _algol_nu_style, _autumn_style, _borland_style, _bw_style,
      _monokai_style, _murphy_style, _native_style, _paraiso_dark_style,
      _paraiso_light_style, _pastie_style, _perldoc_style,  _rrt_style,
      _tango_style, _trac_style, _vim_style, _vs_style, _xcode_style)
+
+
+#
+# Dynamically generated styles
+#
+def make_ansi_style(palette):
+    """Makes an ANSI color style from a color palette"""
+    style = {'NO_COLOR': '0'}
+    for name, t in BASE_XONSH_COLORS.items():
+        closest = find_closest_color(t, palette)
+        if len(closest) == 3:
+            closest = ''.join([a*2 for a in closest])
+        short = rgb2short(closest)[0]
+        style[name] = '38;5;' + short
+        style['BOLD_'+name] = '1;38;5;' + short
+        style['UNDERLINE_'+name] = '4;38;5;' + short
+        style['BOLD_UNDERLINE_'+name] = '1;4;38;5;' + short
+        style['BACKGROUND_'+name] = '48;5;' + short
+    return style
+
+
+def ansi_style_by_name(name):
+    """Gets or makes an ANSI color style by name. If the styles does not
+    exist, it will look for a style using the pygments name.
+    """
+    if name in ANSI_STYLES:
+        return ANSI_STYLES[name]
+    elif not HAS_PYGMENTS:
+        raise KeyError('could not find style {0!r}'.format(name))
+    from xonsh.pygments_cache import get_style_by_name
+    pstyle = get_style_by_name(name)
+    palette = make_palette(pstyle.styles.values())
+    astyle = make_ansi_style(palette)
+    ANSI_STYLES[name] = astyle
+    return astyle
