@@ -8,7 +8,7 @@ import builtins
 import warnings
 
 from xonsh.platform import (best_shell_type, has_prompt_toolkit,
-                            ptk_version_is_supported)
+                            ptk_above_min_supported, ptk_below_max_supported)
 from xonsh.tools import XonshError, print_exception
 from xonsh.events import events
 import xonsh.history.main as xhm
@@ -18,7 +18,7 @@ events.doc('on_transform_command', """
 on_transform_command(cmd: str) -> str
 
 Fired to request xontribs to transform a command line. Return the transformed
-command, or the same command if no transformaiton occurs. Only done for
+command, or the same command if no transformation occurs. Only done for
 interactive sessions.
 
 This may be fired multiple times per command, with other transformers input or
@@ -72,8 +72,8 @@ def transform_command(src, show_diff=True):
                 break
         i += 1
         if i == limit:
-            print_exception('Modifcations to source input took more than '
-                            'the recursion limit number of interations to '
+            print_exception('Modifications to source input took more than '
+                            'the recursion limit number of iterations to '
                             'converge.')
     debug_level = builtins.__xonsh_env__.get('XONSH_DEBUG')
     if show_diff and debug_level > 1 and src != raw:
@@ -132,11 +132,19 @@ class Shell(object):
                 warnings.warn('prompt_toolkit is not available, using '
                               'readline instead.')
                 shell_type = 'readline'
-            elif not ptk_version_is_supported():
+            elif not ptk_above_min_supported():
                 warnings.warn('prompt-toolkit version < v1.0.0 is not '
                               'supported. Please update prompt-toolkit. Using '
                               'readline instead.')
                 shell_type = 'readline'
+            elif not ptk_below_max_supported():
+                warnings.warn('prompt-toolkit version 2.0 is not yet '
+                              'supported. Please see Github PR #2570 for '
+                              'latest status. To use prompt-toolkit now you '
+                              'can downgrade to version 1.x with\n'
+                              'xpip install "prompt_toolkit<2"\n'
+                              'Starting xonsh with readline shell instead.')
+                shell_type = 'readline'
         self.shell_type = env['SHELL_TYPE'] = shell_type
         # actually make the shell
         if shell_type == 'none':
@@ -150,7 +158,7 @@ class Shell(object):
                              shell_type))
         self.shell = shell_class(execer=self.execer,
                                  ctx=self.ctx, **kwargs)
-        # allows history garbage colector to start running
+        # allows history garbage collector to start running
         if hist.gc is not None:
             hist.gc.wait_for_shell = False
 
